/* 2-d example for KOS 1.1.x 
 * AndrewK / Napalm 2001
 * <andrewk@napalm-x.com>
 */

#include <kos.h>
#include <jpeg/jpeg.h>
#include <dcutils/pvrutils.h>

/* font data */
extern char wfont[];

/* textures */
unsigned int font_tex;
unsigned int back_tex;
unsigned int frames_tex;

/* load a binary file into a texture */
void bin_to_texture(char *filename, uint32 tex, int w, int h)
{
    unsigned int fd;
    unsigned short *temp_tex;
    int size;

    temp_tex = ta_txr_map(tex);
    fd = fs_open(filename, O_RDONLY);
    size = fs_total(fd);
    if (size > (w*h*2))
	size = w*h*2;
    fs_read(fd, temp_tex, size);
    fs_close(fd);
}

/* init green guy */
void frames_init()
{
    frames_tex = ta_txr_allocate(512*192*2);
    bin_to_texture("/rd/frames.bin", frames_tex, 512, 192);
}

/* init background */
void back_init()
{
    back_tex = ta_txr_allocate(512*512*2);
    jpeg_to_texture("/rd/background.jpg", back_tex, 512, 1);
}

/* init font */
void font_init()
{
    int i,x,y,c;
    unsigned short * temp_tex;
    
    font_tex = ta_txr_allocate(256*256*2);
    temp_tex = (unsigned short *)malloc(256*128*2);
    
    c = 0;
    for(y = 0; y < 128 ; y+=16)
        for(x = 0; x < 256 ; x+=8) {
            for(i = 0; i < 16; i++) {
                temp_tex[x + (y+i) * 256 + 0] = 0xffff * ((wfont[c+i] & 0x80)>>7);
                temp_tex[x + (y+i) * 256 + 1] = 0xffff * ((wfont[c+i] & 0x40)>>6);
                temp_tex[x + (y+i) * 256 + 2] = 0xffff * ((wfont[c+i] & 0x20)>>5);
                temp_tex[x + (y+i) * 256 + 3] = 0xffff * ((wfont[c+i] & 0x10)>>4);
                temp_tex[x + (y+i) * 256 + 4] = 0xffff * ((wfont[c+i] & 0x08)>>3);
                temp_tex[x + (y+i) * 256 + 5] = 0xffff * ((wfont[c+i] & 0x04)>>2);
                temp_tex[x + (y+i) * 256 + 6] = 0xffff * ((wfont[c+i] & 0x02)>>1);
                temp_tex[x + (y+i) * 256 + 7] = 0xffff * (wfont[c+i] & 0x01);
            }
            c+=16;
        }
    txr_twiddle_copy(temp_tex, 256, 128, font_tex, 256, 256, 0);
}

/* draw green guy */
void draw_guy(float x1, float y1, float z1, float a, float r, float g, float b,int c, float xs, float ys)
{
    poly_hdr_t poly;
    vertex_ot_t     vert;
    int             ix, iy;
    float           u1, v1, u2, v2;
    
    ta_poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB1555, 512, 256, frames_tex, TA_BILINEAR_FILTER);
    ta_commit_poly_hdr(&poly);
    
    ix = (c % 8) * 64;
    iy = (c / 8) * 64;
    u1 = ix * 1.0f / 512.0f;
    v1 = iy * 1.0f / 256.0f;
    u2 = (ix+64) * 1.0f / 512.0f;
    v2 = (iy+64) * 1.0f / 256.0f;
    
    vert.flags = TA_VERTEX_NORMAL;
    vert.x = x1;
    vert.y = y1 + 64.0f * ys;
    vert.z = z1;
    vert.u = u1;
    vert.v = v2;
    vert.dummy1 = vert.dummy2 = 0;
    vert.a = a;
    vert.r = r;
    vert.g = g;
    vert.b = b;
    vert.oa = vert.or = vert.og = vert.ob = 0.0f;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1;
    vert.y = y1;
    vert.u = u1;
    vert.v = v1;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1 + 64.0f * xs;
    vert.y = y1 + 64.0f * ys;
    vert.u = u2;
    vert.v = v2;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.flags = TA_VERTEX_EOL;
    vert.x = x1 + 64.0f * xs;
    vert.y = y1;
    vert.u = u2;
    vert.v = v1;
    ta_commit_vertex(&vert, sizeof(vert));
}

/* draw background */
void draw_back(void)
{
    poly_hdr_t poly;
    vertex_ot_t vert;

    ta_poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565_TWID, 512, 512, back_tex, TA_BILINEAR_FILTER);
    ta_commit_poly_hdr(&poly);
    
    vert.r = vert.g = vert.b = 1.0f;
    vert.a = 1.0f;
    vert.oa = vert.or = vert.ob = 0.0f;
    vert.flags = TA_VERTEX_NORMAL;
    
    vert.x = 1;
    vert.y = 1;
    vert.z = 1;
    vert.u = 0.0;
    vert.v = 0.0;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = 640;
    vert.y = 1;
    vert.z = 1;
    vert.u = 1.0;
    vert.v = 0.0;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = 1;
    vert.y = 480;
    vert.z = 1;
    vert.u = 0.0;
    vert.v = 1.0;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = 640;
    vert.y = 480;
    vert.z = 1;
    vert.u = 1.0;
    vert.v = 1.0;
    vert.flags = TA_VERTEX_EOL;
    ta_commit_vertex(&vert, sizeof(vert));
}

/* draw one character */
void draw_char(float x1, float y1, float z1, float a, float r, float g, float b, int c, float xs, float ys)
{
    vertex_ot_t     vert;
    int             ix, iy;
    float           u1, v1, u2, v2;
    
    ix = (c % 32) * 8;
    iy = (c / 32) * 16;
    u1 = (ix + 0.5f) * 1.0f / 256.0f;
    v1 = (iy + 0.5f) * 1.0f / 256.0f;
    u2 = (ix+7.5f) * 1.0f / 256.0f;
    v2 = (iy+15.5f) * 1.0f / 256.0f;
    
    vert.flags = TA_VERTEX_NORMAL;
    vert.x = x1;
    vert.y = y1 + 16.0f * ys;
    vert.z = z1;
    vert.u = u1;
    vert.v = v2;
    vert.dummy1 = vert.dummy2 = 0;
    vert.a = a;
    vert.r = r;
    vert.g = g;
    vert.b = b;
    vert.oa = vert.or = vert.og = vert.ob = 0.0f;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1;
    vert.y = y1;
    vert.u = u1;
    vert.v = v1;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1 + 8.0f * xs;
    vert.y = y1 + 16.0f * ys;
    vert.u = u2;
    vert.v = v2;
    ta_commit_vertex(&vert, sizeof(vert));
    
    vert.flags = TA_VERTEX_EOL;
    vert.x = x1 + 8.0f * xs;
    vert.y = y1;
    vert.u = u2;
    vert.v = v1;
    ta_commit_vertex(&vert, sizeof(vert));
}

/* draw a string */
void draw_string(float x, float y, float z, float a, float r, float g, float b, char *str, float xs, float ys) {
  poly_hdr_t poly;
  ta_poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB4444_TWID, 256, 256,
		  font_tex, TA_BILINEAR_FILTER);
  ta_commit_poly_hdr(&poly);
  
  while (*str) {
    draw_char(x, y, z, a, r, g, b, *str++, xs, ys);
	x+=8*xs;
  }
}

/* draw a horizontally centered string */
void draw_string_centered(float y, float z, float a, float r, float g, float b, char *str, float xs, float ys) {
  poly_hdr_t poly;
  float x;

  x = 320.0f - strlen(str)*4*xs;

  ta_poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB4444_TWID, 256, 256, font_tex, TA_BILINEAR_FILTER);
  ta_commit_poly_hdr(&poly);
  
  while (*str) {
    draw_char(x, y, z, a, r, g, b, *str++, xs, ys);
	x+=8*xs;
  }
}

/* base y coordinate */
int y = 0;

/* sizes the green guy is drawn at */
float sizes[100];

/* draw one frame */
void draw_frame(void)
{
    ta_begin_render();
    draw_back();
    ta_commit_eol();
    draw_string_centered(y % 480, 3, 1, 0,0,0, "2-D EXAMPLE FOR KOS 1.1.X", 2, 2);
    draw_string_centered((y + 40) % 480, 3, 1, 0,0,0, "ANDREWK / NAPALM 2001      <ANDREWK@NAPALM-X.COM>", 1, 1);
    draw_string_centered((y + 80) % 480, 3, 1, 0,0,0, "THE BACKGROUND IS A TEXTURE LOADED FROM A JPEG", 1, 1);
    draw_string_centered((y + 120) % 480, 3, 1, 0,0,0, "THE GREEN GUY IS A TEXTURE LOADED FROM A BINARY FILE", 1, 1);
    draw_string_centered((y + 160) % 480, 3, 1, 0,0,0, "THE FONT DATA IS LINKED INTO THE PROGRAM", 1, 1);
    draw_string_centered((y + 220) % 480, 3, 1, 0,0,0, "HERES SOME COLORED TEXT:", 1, 1);
    draw_string(152, (y + 240) % 480, 3, 1, 1,0,0, "RED", 3, 3);
    draw_string(248, (y + 240) % 480, 3, 1, 0,1,0, "GREEN", 3, 3);
    draw_string(392, (y + 240) % 480, 3, 1, 0,0,1, "BLUE", 3, 3);
    draw_string_centered((y + 320) % 480, 3, 1, 0,0,0, "PRESS START TO EXIT", 1, 1);
    draw_guy((y % 640) - (1 + sizes[y%100])*32, (y % 480) - (1 + sizes[y%100])*32, 2, 1, 1,1,1, (y/2) % 22, 1+sizes[y%100], 1+sizes[y%100]);
    ta_commit_eol();
    ta_finish_frame();

    y++;
}

/* romdisk */
extern uint8 romdisk_boot[];

int main(void)
{
    int i;
    cont_cond_t cond;

    /* init kos  */
    kos_init_all(IRQ_ENABLE | TA_ENABLE | THD_ENABLE, romdisk_boot);
    
    /* init font */
    font_init();
    
    /* init background */
    back_init();

    /* init frames */
    frames_init();

    /* setup sizes for green guy */
    for(i=0; i<50; i++)
	sizes[i] = i/25.0;
    for(i=50; i<100; i++)
	sizes[i] = sizes[99 - i];

    /* keep drawing frames until start is pressed */
    while(1) {
        if (cont_get_cond(maple_first_controller(), &cond) < 0)
            break;
        
        if (!(cond.buttons & CONT_START))
            break;

	draw_frame();
    }

    /* shutdown kos */
    kos_shutdown_all();

    return 0;
}

